/*
** Module   : OutputExtentField.java
** Abstract : Data wrapper which associates with a DMO extent property
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20130209 Created initial version.
*/

package com.goldencode.p2j.persist.type;

import com.goldencode.p2j.persist.DataModelObject;
import com.goldencode.p2j.persist.FieldReference;
import com.goldencode.p2j.util.BaseDataType;
import com.goldencode.p2j.util.OutputExtentParameter;

/**
 * Data wrapper which associates with a DMO extent property.  An instance of this class is used to
 * wrap the extent field when passed to an output or input-output parameter.  It manages the 
 * hidden assignment of an input-output or an output parameter to the database field with which it
 * is associated, upon return from a procedure or function.
 */
public class OutputExtentField<T extends BaseDataType>
extends OutputExtentParameter<T>
{
   /** The field reference instance for. */
   private final FieldReference fieldRef;

   /**
    * Constructor.
    * 
    * @param   dmo
    *          Proxy for record whose field is being accessed.
    * @param   property
    *          Name of dmo field.
    */
   public OutputExtentField(DataModelObject dmo, String property)
   {
      this(dmo, property, false);
   }

   /**
    * Constructor.
    * 
    * @param   dmo
    *          Proxy for record whose field is being accessed.
    * @param   property
    *          Name of dmo field.
    * @param   initialize
    *          Should this variable be initialized with the DMO's current
    *          value for the target field?
    */
   public OutputExtentField(DataModelObject dmo,
                            String          property,
                            boolean         initialize)
   {
      // this needs to register with the TM before the FieldAssigner instance
      super();
      
      this.fieldRef = new FieldReference(dmo, property, null);
   }

   /**
    * Get the current extent reference.
    * 
    * @return   See above.
    */
   public T[] getVariable()
   {
      return (T[]) fieldRef.getExtent();
   }

   /**
    * Change the caller's reference to the new reference.
    * 
    * @param    reference
    *           The new reference.
    */
   public void setVariable(T[] reference)
   {
      fieldRef.set(reference);
   }

}
